/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.gui.FilmStrip;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;
import javax.swing.border.EmptyBorder;

/**
 * Displays a dialog asking the user to input a username, email, and any comments about the snapshot. 
 * @author Chris Callendar
 */
class SnapshotDialog extends JDialog {

	private SnapShot snapShot;
	
	/** Creates new form JDialog */
	public SnapshotDialog(Frame parent, String title, boolean modal, SnapShot snapShot) {
		super(parent, title, modal);
		this.snapShot = snapShot;
		initComponents();
	}

	private void initComponents() {
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		setTitle("Snapshot Comments");
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent evt) {
				closeDialog();
			}
		});

		JPanel jPanel1 = new JPanel();
		jPanel1.setLayout(new BoxLayout(jPanel1, BoxLayout.Y_AXIS));

		JPanel userNamePanel = new JPanel();
		userNamePanel.setLayout(new BoxLayout(userNamePanel, BoxLayout.X_AXIS));
		userNamePanel.setBorder(new EmptyBorder(new Insets(15, 5, 5, 10)));
		JLabel userName = new JLabel();
		userName.setText("User name:");
		userName.setPreferredSize(new Dimension(80, 25));
		userNamePanel.add(userName);

		final JTextField userNameField = new JTextField();
		userNameField.setMinimumSize(new Dimension(50, 20));
		userNameField.setPreferredSize(new Dimension(250, 25));
		userNamePanel.add(userNameField);

		jPanel1.add(userNamePanel);
		JPanel jPanel3 = new JPanel();
		jPanel3.setLayout(new BoxLayout(jPanel3, BoxLayout.X_AXIS));
		jPanel3.setBorder(new EmptyBorder(new Insets(10, 5, 5, 10)));
		JLabel emailLabel = new JLabel();
		emailLabel.setText("Email:");
		emailLabel.setPreferredSize(new Dimension(80, 25));
		jPanel3.add(emailLabel);

		final JTextField emailField = new JTextField();
		emailField.setMinimumSize(new Dimension(50, 20));
		emailField.setPreferredSize(new Dimension(250, 25));
		jPanel3.add(emailField);

		jPanel1.add(jPanel3);
		JPanel jPanel4 = new JPanel();
		jPanel4.setLayout(new BorderLayout());
		jPanel4.setBorder(new EmptyBorder(new Insets(2, 5, 2, 5)));
		JLabel commentLabel = new JLabel();
		commentLabel.setHorizontalAlignment(SwingConstants.LEFT);
		commentLabel.setText("Comments:");
		commentLabel.setPreferredSize(new Dimension(200, 30));
		jPanel4.add(commentLabel, BorderLayout.CENTER);
		jPanel1.add(jPanel4);

		getContentPane().add(jPanel1, BorderLayout.NORTH);

		JPanel okPanel = new JPanel();
		JButton okButton = new JButton();
		okPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
		okPanel.setBorder(new EmptyBorder(new Insets(2, 2, 2, 2)));
		okButton.setText("OK");
		okPanel.add(okButton, new GridBagConstraints());

		getContentPane().add(okPanel, BorderLayout.SOUTH);

		final JTextArea commentArea = new JTextArea();
		commentArea.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
		JScrollPane scroll = new JScrollPane(commentArea);
		scroll.setPreferredSize(new Dimension(500, 200));
		getContentPane().add(scroll, BorderLayout.CENTER);

	   	okButton.addActionListener(new ActionListener() {
			   public void actionPerformed(ActionEvent evt) {
					saveSnapshot(userNameField.getText().trim(), emailField.getText().trim(), commentArea.getText().trim());
					closeDialog();
			   }
		});

		userNameField.setText(ShrimpSnapShot.USER_NAME);
		emailField.setText(ShrimpSnapShot.DEFAULT_USER_EMAIL);

		pack();
		commentArea.requestFocus(true);
	}
	
	private void saveSnapshot(String name, String email, String comment) {
		ShrimpSnapShot.USER_NAME = (name.length() == 0 ? System.getProperty("user.name") : name);
		ShrimpSnapShot.USER_EMAIL = (email.length() == 0 ? ShrimpSnapShot.DEFAULT_USER_EMAIL : email);
		snapShot.setComment(comment);
	}	

	/** Closes the dialog */
	private void closeDialog() {
		setVisible(false);
		dispose();
	}
	
}