/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.jambalaya;

import java.awt.event.ContainerAdapter;
import java.awt.event.ContainerEvent;
import java.net.URI;

import javax.swing.JFrame;
import javax.swing.JTabbedPane;

import ca.uvic.csr.shrimp.DataBean.DataBean;
import ca.uvic.csr.shrimp.ShrimpApplication.BeanNotFoundException;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.gui.ProgressDialog;
import edu.stanford.smi.protege.event.FrameEvent;
import edu.stanford.smi.protege.event.FrameListener;
import edu.stanford.smi.protege.model.Project;
import edu.stanford.smi.protege.plugin.AbstractProjectPlugin;
import edu.stanford.smi.protege.ui.ProjectManager;
import edu.stanford.smi.protege.ui.ProjectMenuBar;
import edu.stanford.smi.protege.ui.ProjectToolBar;
import edu.stanford.smi.protege.ui.ProjectView;

/**
 * Jambalaya, in addition to being a "tab widget" plug-in, is
 * also a Protege "project" plug-in. This allows Jambalaya actions
 * and functionality to show and be invoked in Protege even when
 * the Jambalaya tab is not showing. 
 * 
 * @author Rob Lintern, Chris Callendar
 */
public class JambalayaProjectPlugin extends AbstractProjectPlugin {
	
	private static JambalayaProjectPlugin jambalayaProjectPlugin;
	private JambalayaApplication jambalayaApplication;
	private static ProjectView projectView;
	private boolean mainWindowListenerAdded = false;
    private JambalayaProject jambalayaProject;
    private Project protegeProject;
	
    private FrameListener frameListener = new FrameListener () {
        public void visibilityChanged(FrameEvent event) {
            knowledgeBaseChanged();       
       }        
       public void ownSlotValueChanged(FrameEvent event) {
           knowledgeBaseChanged();       
       }        
       public void ownSlotRemoved(FrameEvent event) {
           knowledgeBaseChanged();       
       }
       public void ownSlotAdded(FrameEvent event) {
           knowledgeBaseChanged();       
       }
       public void ownFacetValueChanged(FrameEvent event) {
           knowledgeBaseChanged();       
       }
       public void ownFacetRemoved(FrameEvent event) {
           knowledgeBaseChanged();       
       }
       public void ownFacetAdded(FrameEvent event) {
           knowledgeBaseChanged();       
       }
       public void nameChanged(FrameEvent event) {
           knowledgeBaseChanged();       
       }
       public void deleted(FrameEvent event) {
           knowledgeBaseChanged();       
       }
       public void browserTextChanged(FrameEvent event) {
           knowledgeBaseChanged();       
       }
   };
    
    public JambalayaProjectPlugin() {
        super();
        jambalayaProjectPlugin = this;
    }
    
    public static JambalayaProjectPlugin getJambalayaProjectPlugin() {
        return jambalayaProjectPlugin;
    }
    
    /**
     * Gets the JambalayaApplication. It will be created if it hasn't been yet.
     * @return
     */
    public JambalayaApplication getJambalayaApplication() {
    	return getJambalayaApplication(null);
    }
    
    public JambalayaApplication getJambalayaApplication(String propertiesFile) {
    	if (jambalayaApplication == null) {
    		if (propertiesFile != null) {
    			jambalayaApplication = new JambalayaApplication(propertiesFile);
    		} else {
    			jambalayaApplication = new JambalayaApplication();
    		}
		}
    	return jambalayaApplication;
    }

    /**
     * Gets the current JambalayaProject.  It will be created if it is null.
     */
    public JambalayaProject getJambalayaProject() {
    	if (jambalayaProject == null) {
    		Project project = getProtegeProject();
        	URI uri = (project != null ? project.getProjectURI() : null);
            jambalayaProject = getJambalayaApplication().createJambalayaProject(project, uri);
    	}
    	return jambalayaProject;
    }
    
    /**
     * Used by the Jambalaya applets.
     */
    public void setJambalayaProject(JambalayaProject project) {
    	this.jambalayaProject = project;
    }
    
    /**
     * Sets the protect project and adds the frame listener.
     * Also used by the Jambalaya applets.
     */
    public void setProtegeProject(Project project) {
    	if ((project == null) && (protegeProject != null) && (protegeProject.getKnowledgeBase() != null)) {
    		// remove the listener
    		protegeProject.getKnowledgeBase().removeFrameListener(frameListener);
    	}
    	
    	protegeProject = project;
    	if (protegeProject != null) {
   	 		// watch for any changes in the knowledgebase
    		protegeProject.getKnowledgeBase().addFrameListener(frameListener);
    	}
    }
    
    public Project getProtegeProject() {
    	if (protegeProject == null) {
    		setProtegeProject(ProjectManager.getProjectManager().getCurrentProject());
    	}
    	return protegeProject;
    }
    
    public void afterCreate(Project p) {     
        afterCreateOrLoad(p);
    }

    public void afterLoad(Project p) {
    	afterCreateOrLoad(p);
    }
    
    private void afterCreateOrLoad(Project project) {
    	setProtegeProject(project);
    }
    
    private void knowledgeBaseChanged() {
        //System.out.println("knowledgeBaseChanged");
        JambalayaProject project = getJambalayaProject();
        if (project != null) {
            JambalayaTab jambalayaTab = getJambalayaTab();
            if (jambalayaTab != null) {
            	// put an asterisk in the name of the jambalaya tab to indicate that it needs to be refreshed
                jambalayaTab.setLabel("Jambalaya *"); 
                JTabbedPane tabbedPane  = (JTabbedPane) jambalayaTab.getParent();                
                tabbedPane.setTitleAt(tabbedPane.indexOfComponent(jambalayaTab), "Jambalaya *");   
            }
            try {
                DataBean dataBean = (DataBean) project.getBean(ShrimpProject.DATA_BEAN);
                dataBean.setDataIsDirty();
                dataBean.clearBufferedData();
            } catch (BeanNotFoundException e) {
            	// @tag Jambalaya(clientserver) : this was happening in client server mode after the project was closed I think
                //e.printStackTrace();
            }
        }
    }
    
    void jambalayaProjectRefreshed() {
        JambalayaTab jambalayaTab = getJambalayaTab();
        if (jambalayaTab != null) {
            jambalayaTab.setLabel("Jambalaya"); // remove asterisk from the name of the jambalaya tab
            JTabbedPane tabbedPane  = (JTabbedPane) jambalayaTab.getParent();           
            tabbedPane.setTitleAt(tabbedPane.indexOfComponent(jambalayaTab), "Jambalaya");            
        }
    }
    
    public void afterShow(ProjectView view, ProjectToolBar toolBar, ProjectMenuBar menuBar) {
        projectView = view;
        
		// @tag Jambalaya(Plugin(afterCreate)) : ensure that the Protege Project is set
        // This is a bug in Protege - afterCreate() doesn't get called (has been fixed in the newest version)
       	getProtegeProject();
        
        // at this point we have some reference to a gui object, so lets find its top level ancestor
        JFrame mainFrame = (JFrame)view.getTopLevelAncestor();
        final JambalayaApplication app = getJambalayaApplication();
		app.setParentFrame(mainFrame);
		ProgressDialog.createProgressDialog(mainFrame, app.getName() + " Progress ...");
		
		if (!mainWindowListenerAdded) {
			mainFrame.addContainerListener(new ContainerAdapter () {
				public void componentRemoved(ContainerEvent e) {
					if (e.getContainer().getComponentCount() == 0) {
						// at the moment this is the best way to know if protege is about to close
						// System.exit(0) is called by protege before a windowing closing event thrown
					    app.close();
					}
				}
			});
	    	mainWindowListenerAdded = true;
		}

    }

    public void beforeSave(Project p) {
    	// do nothing
    }
    
    public void afterSave(Project p) {
    	// do nothing
    }

    public void beforeHide(ProjectView view, ProjectToolBar toolBar, ProjectMenuBar menuBar) {
    	// take the closing snapshot - it must be done here and NOT in beforeClose because
    	// by then the gui has already been removed
    	JambalayaTab tab = getJambalayaTab();
    	if (tab != null) {
    		tab.getContentPane().takeSnapShot(getJambalayaProject());
    	}
    }
    
    public void beforeClose(Project p) {
    	// we don't want to call getJambalayaProject() because a project might not have been created yet
    	// when a new Protege Project is created an NP exception is thrown otherwise
       	if (jambalayaProject != null) {
       		jambalayaProject.disposeProject();
       	}
        setProtegeProject(null);
        jambalayaProject = null;
    }
    
    public static JambalayaTab getJambalayaTab() {
        JambalayaTab jambalayaTab = null;
        if (projectView != null) {
            jambalayaTab = (JambalayaTab)projectView.getTabByClassName(JambalayaTab.class.getName());
        }
        return jambalayaTab;
    }
    
	public static boolean isOwlProject(Project project) {
	    boolean isOwlProject = false;
		try {
		    isOwlProject = OWLSpecificUtils.isOwlProject(project);
        } catch (Throwable e) {
			// an exception is thrown if there is no protege-owl plugin installed
        }
	    return isOwlProject;
	}
    

}
